<?php

// Author: oxana.smirnova@quark.lu.se

require_once('headfoot.inc');
require_once('lmtable.inc');
require_once('comfun.inc');
require_once('cnvtime.inc');
require_once('cnvname.inc');
require_once('toreload.inc');
require_once('ldap_nice_dump.inc');

// Setting up the page itself

$toppage = new LmDoc("quelist");

/**
 * @return int
 * @param host string
 * @param port int
 * @param qname string
 * @desc Retrieves the queue information (job list)
 */
function do_quelist ($host="grid.quark.lu.se",$port=2135,$qname="default")
{
  /**
   * FUNCTION DO_QUELIST (string host, int port, string qname)
   * Retrieves the queue job list information from a given NorduGrid domain
   * Uses LDAP functions of PHP
   * 
   * Author: O.Smirnova (June 2002)
   * inspired by the LDAPExplorer by T.Miao
   * 
   * input:
   * host (default: grid.quark.lu.se)
   * port (default: 2135)
   * qname (default: "default")
   * 
   * output:
   * an HTML table, containing the list of the jobs in the queue
   */
   
  $lim = array( "dn", JOB_NAME, JOB_GOWN, JOB_SUBM, JOB_STAT, JOB_USET, JOB_USEM, JOB_ERRS );
  
  $tlim = 10;
  $tout = 10;

  // Header table

  $clstring=popup("clusdes.php?host=$host&port=$port",700,620,1);

  $toptit   = "<b><i>Queue $qname at <a href=\"$clstring\">$host</a></i></b>";
  $toptable = new LmTableTop("","$toptit");

  // ldapsearch filter strings for cluster and queues
  
  $filstr = "(&(objectclass=".OBJ_AJOB.")(".JOB_EQUE."=$qname))";
  $dn     = DN_LOCAL;
  $topdn  = DN_GLOBL;
  
  // Establish connection to the requested LDAP server
  
  $ds = ldap_connect($host,$port);
  if ($ds) {
    
    // If contact OK, search for NorduGrid clusters
    
    $basedn = QUE_NAME."=".$qname.",".CLU_NAME."=".$host.",";
    $locdn  = $basedn.$dn;
    $aaa = ldap_nice_dump($ds,$locdn);
    echo "<BR>";    
    
    $sr = ldap_search($ds,$dn,$filstr,$lim,0,0,$tlim,LDAP_DEREF_NEVER,$tout);
    // Fall back to conventional LDAP
    if (!$sr) $sr = ldap_search($ds,$dn,$filstr,$lim,0,0,$tlim,LDAP_DEREF_NEVER);

    if ($sr) {
      $nmatch = ldap_count_entries($ds,$sr);
      if ($nmatch > 0) {
        $entries = ldap_get_entries($ds,$sr);
        $njobs   = $entries["count"];
        
        define("CMPKEY",JOB_SUBM);
        usort($entries,"ldap_entry_comp");
        
        // HTML table initialisation
        
        $ltable = new LmTable("quelist");
        
        // loop on jobs
        
        for ($i=1; $i<$njobs+1; $i++) {
          $jobdn   = rawurlencode($entries[$i]["dn"]);
          $curstat = $entries[$i][JOB_STAT][0];
          $stahead = substr($curstat,0,12);
          if ($stahead=="FINISHED at:") {
            $ftime   = substr(strrchr($curstat, " "), 1);
            $ftime   = cnvtime($ftime);
            $curstat = "FINISHED at: ".$ftime;
          }
          $uname    = $entries[$i][JOB_GOWN][0];
          $encuname = rawurlencode($uname);
	  $family   = cnvname($uname);
          
          $jobname = ($entries[$i][JOB_NAME][0]) ? $entries[$i][JOB_NAME][0] : "<font color=red>N/A</font>";
          $time    = ($entries[$i][JOB_USET][0]) ? $entries[$i][JOB_USET][0] : "";
          $memory  = ($entries[$i][JOB_USEM][0]) ? $entries[$i][JOB_USEM][0] : "";
	  $error   = ($entries[$i][JOB_ERRS][0]);
	  if ( $error ) $error = ( preg_match("/user/i",$error) ) ? "X" : "!";
	  $status  = "All";
          $newwin  = popup("jobstat.php?host=$host&port=$port&status=$status&jobdn=$jobdn",750,430,4);
          $usrwin  = popup("userlist.php?bdn=$topdn&owner=$encuname",700,500,5);
          
          // filling the table
          
          $lrowcont[] = "$i&nbsp;<b><font color=red>$error</font></b>";
          $lrowcont[] = "<a href=\"$newwin\">$jobname</a>";
          $lrowcont[] = "<a href=\"$usrwin\">$family</a>";
          $lrowcont[] = "$curstat";
          $lrowcont[] = "$time";
          $lrowcont[] = "$memory";
          $ltable->addrow($lrowcont);
          $lrowcont = array ();
        }
        $ltable->close();
      } else {
        echo "<BR><font color=red><B>No Grid jobs found</B></font>\n";
        return 1;
      }
    } else {
      echo "<BR><font color=red><B>No information found</B></font>\n";
      return 1;
    }
    ldap_free_result($sr);
    ldap_close($ds);
    return 0;
  }
  else {
    echo "<BR><font color=red><b>Server unavailable</b></font>\n";
    return 1;
  }
}

$host   = $_GET["host"];
$port   = $_GET["port"];
$qname  = $_GET["qname"];
do_quelist($host,$port,$qname);

// Done

$toppage->close();

?>
